from .record import Record
from .types.category import Category
from .types.reason import Reason


class Exclusion(Record):
    EXCLUSION_METRICS = {
        "19": "excluded",
        "20": "name",
        "21": "reason",
        "22": "description"
    }

    def __init__(
            self,
            record_key,
            record_id,
            reason=None,
            excluded=None,
            name=None,
            description=None,
            container=None
    ):
        super().__init__(record_key, record_id, Category.EXCLUSION, name=name, container=container)

        if reason and not isinstance(reason, Reason):
            raise Exception('reason arg for record {} must be an instance of Reason enum'
                            .format(record_key))

        self._reason = reason
        self._excluded = excluded
        self._description = description

    @staticmethod
    def from_dict(exclusion_dict):
        record_id = exclusion_dict.get('key')
        excluded = exclusion_dict.get('excluded')
        name = exclusion_dict.get('name')
        description = exclusion_dict.get('description')
        reason = Reason.get_name(exclusion_dict.get('reason'))

        return Exclusion(
            record_key=record_id,
            record_id=record_id,
            reason=reason,
            excluded=excluded,
            name=name,
            description=description
        )

    @staticmethod
    def from_databrary(exclusion_dict):
        record_id = exclusion_dict.get('id')
        measures = exclusion_dict.get('measures')

        if not measures:
            return Exclusion(record_key=record_id, record_id=record_id)

        excluded = measures.get('19')
        name = measures.get('20')
        description = measures.get('22')
        reason = Reason.get_name(measures['21'])

        return Exclusion(
            record_key=record_id,
            record_id=record_id,
            reason=reason,
            excluded=excluded,
            name=name,
            description=description
        )

    def get_template(self):
        return {
            "key": "{}".format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
            "reason": "",
            "excluded": "",
            "description": ""
        }

    def to_dict(self):
        result = {
            "key": "{}".format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
        }

        if self.get_reason() is not None:
            result['reason'] = self.get_reason().value

        if self.get_excluded() is not None:
            result["excluded"] = self.get_excluded()

        if self.get_description() is not None:
            result["description"] = self.get_description()

        return result

    def get_reason(self):
        return self._reason

    def get_excluded(self):
        return self._excluded

    def get_description(self):
        return self._description



from enum import Enum


class Term(Enum):
    FULL_TERM = "Full term",
    PRETERM = "Preterm"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        return cls._value2member_map_[value]



import os
import warnings

from ..databrary.recordFactory import RecordFactory
from ..databrary.asset import Asset
from ..databrary.volume import Volume

from ..utils import utils
from ..databrary.container import Container
from ..databrary.types.category import Category
from ..api.pybrary import Pybrary
from .. import constants


class Curation:
    @staticmethod
    def from_databrary(volume_id, username=None, password=None, superuser=False):
        pb = Pybrary.get_instance(username, password, superuser)
        volume_data = pb.get_volume_info(volume_id)

        return Volume.from_databrary(volume_data)

    @staticmethod
    def from_files(volume_name, containers_file, assets_file=None, records_file=None):
        if volume_name is None:
            raise Exception('You must provide a valid volume name.')

        if containers_file is None:
            raise Exception('Containers file is required')

        containers = Curation.from_csv(containers_file)

        assets = dict()
        if assets_file is None:
            warnings.warn('Assets found in the containers file will be ignored. PLease provide an assets file')
        else:
            assets = Curation.from_csv(assets_file, 'id')

        records = dict()
        if records_file is None:
            warnings.warn('Records found in the containers file will be ignored. PLease provide a records file')
        else:
            records = Curation.from_csv(records_file)

        for key, container_dict in containers.items():
            asset_ids = container_dict.get('assets')
            assets_map = dict()
            for idx, asset_id in enumerate(asset_ids):
                if asset_id not in assets:
                    warnings.warn("Asset id {} not find in asset file, it will be ignored".format(asset_id))
                    continue

                assets_map[asset_id] = assets.get(asset_id)

            container_dict['assets'] = list(assets_map.values())

            record_ids = container_dict.get('records')
            records_map = dict()
            for idx, record_id in enumerate(record_ids):
                if record_id not in records:
                    warnings.warn("record id {} not find in record file, it will be ignored".format(record_id))
                    continue

                records_map[record_id] = records.get(record_id)

            container_dict['records'] = list(records_map.values())
            containers[key] = Container.from_dict(container_dict)

        return Volume(volume_name, containers)

    @staticmethod
    def to_files(volume, containers_output, assets_output, records_output):
        assets = volume.get_assets()
        Curation.map_to_csv(assets, assets_output)

        records = volume.get_records()
        Curation.map_to_csv(records, records_output)

        containers = volume.get_containers()
        container_list = []
        for container in containers.values():
            container_dict = container.to_dict()
            container_dict["assets"] = list(map(lambda asset: asset.get('id'), container_dict.get('assets')))
            container_dict["records"] = list(map(lambda record: record.get('key'), container_dict.get('records')))
            container_list.append(container_dict)

        Curation.list_to_csv(container_list, containers_output)

    @staticmethod
    def to_json(volume, output):
        if output is None:
            raise Exception('Please provide an output path')

        if utils.get_file_extension(output) != 'json':
            raise Exception('Ingest files must be in JSON format, ingest file cannot be saved in {}'
                            .format(output))

        utils.dump_into_json(volume.to_dict(), output)

    @staticmethod
    def generate_assets(folder, output=None):
        """
        Find all supported files in a folder (recursively) adds clips for video files.
        return the following structure
            [
                {
                    "key": 1
                    "release": null,
                    "position": "auto",
                    "name": "FILE_NAME,
                    "file": "FILE_PATH_IN_DATABRARY_STAGING_FOLDER"
                }
            ]
        :param output: dump assets list into a csv file
        :param folder: Folder path where to lookup for assets
        :return: a List of dict with supported assets found in folder
        """
        if not os.path.isdir(folder):
            raise Exception('{} is not a directory'.format(folder))

        print('Parsing {}'.format(os.path.abspath(folder)))

        assets = dict()
        idx = 0
        for root, dirs, files in os.walk(folder):
            for file in files:
                try:
                    assets[idx] = Asset(file_path=os.path.join(root, file), asset_id=idx)
                    idx += 1
                except Exception as e:
                    warnings.warn("Error while generating asset list: {} ".format(e))

        if output is not None:
            Curation.map_to_csv(assets, output, template=True)
            print('Assets printed in {}'.format(os.path.abspath(output)))

        return assets

    @staticmethod
    def generate_records(categories=None, output=None):
        records = dict()

        idx = 0
        for category, value in categories.items():
            if not Category.has_value(category):
                raise Exception('Category {} is not valid, it will be ignored'.format(category))

            if int(value) < 1:
                raise Exception('The value {} for {} is not valid, it must be > 0'.format(value, category))

            for i in range(value):
                records[idx] = RecordFactory.from_dict({
                        "category": category,
                        "key": idx
                    })
                idx = idx + 1

        if output is not None:
            Curation.map_to_csv(records, output, template=True)
            print('Records printed in {}'.format(os.path.abspath(output)))

        return records

    @staticmethod
    def generate_containers(value=1, output=None):
        if int(value) < 1:
            raise Exception('The value {} is not valid, it must be > 0'.format(value))

        containers = {i: Container(container_key=i, container_id=i) for i in range(value)}

        if output is not None:
            Curation.map_to_csv(containers, output, template=True)
            print('Containers printed in {}'.format(os.path.abspath(output)))

        return containers

    @staticmethod
    def map_to_csv(data_map, output, template=False):
        data_list = [data.get_template() if template else data.to_dict() for data in data_map.values()]
        Curation.list_to_csv(data_list, output)

    @staticmethod
    def list_to_csv(data_list, output):
        utils.dump_into_csv(data_list, output)

    @staticmethod
    def from_csv(file_path, id_name="key"):
        data = dict()

        for data_dict in utils.read_csv(file_path):
            data_id = data_dict.get(id_name)
            if data_id in data:
                raise Exception('Found duplicate key {} in {}, the container {} will be ignored'
                                .format(data_id, file_path, data_dict.get('name')))

            if data_dict.get('assets'):
                assets_str = data_dict.get('assets')
                assets_splits = assets_str.replace('[', '').replace(']', '').split(',')
                assets_list = list(
                    map(
                        lambda s: int(s.strip()) if len(s.strip()) > 0 else None,
                        assets_splits
                    )
                )
                data_dict['assets'] = assets_list

            if data_dict.get('records'):
                records_str = data_dict.get('records')
                records_splits = records_str.replace('\'', '').replace('[', '').replace(']', '').split(',')
                records_list = list(
                    map(
                        lambda s: int(s.strip()) if len(s.strip()) > 0 else None,
                        records_splits
                    )
                )
                data_dict['records'] = records_list

            data[data_id] = data_dict

        return data

    @staticmethod
    def generate_sql_query(volume_id, suffix_path=constants.DEFAULT_SERVER_PREFIX):
        """
        Generate Databrary DB query that
        :param suffix_path:
        :param volume_id: Volume ID
        :return:
        """
        return "COPY (" \
               "select 'mkdir -p {}/{}/' || sa.container || ' && ' || E'cp \"/nyu/store/' || substr(cast(sha1 as varchar(80)), 3, 2) || '/' || right(cast(sha1 as varchar(80)), -4) || '\" \"' || '/nyu/stage/reda/{}/' || sa.container || '/' || CASE WHEN a.name LIKE '%.___' IS FALSE THEN a.name || '.' || f.extension[1] || '\"' ELSE a.name || '\"' END from slot_asset sa inner join asset a on sa.asset = a.id inner join format f on a.format = f.id where a.volume = {}) TO '/tmp/volume_{}.sh';".format(suffix_path, volume_id, volume_id, volume_id)



from .record import Record
from .types.category import Category


class Pilot(Record):
    PILOT_METRICS = {
        "16": "pilot",
        "17": "name",
        "18": "description"
    }

    def __init__(self, record_key, record_id, name=None, pilot=None, description=None, container=None):
        super().__init__(record_key, record_id, Category.PILOT, name=name, container=container)
        self.__pilot = pilot
        self.__description = description

    @staticmethod
    def from_dict(pilot_dict):
        record_id = pilot_dict.get('key')
        name = pilot_dict.get('name')
        pilot = pilot_dict.get('pilot')
        description = pilot_dict.get('description')
        return Pilot(record_key=record_id, record_id=record_id, name=name, pilot=pilot, description=description)

    @staticmethod
    def from_databrary(pilot_dict):
        record_id = pilot_dict.get('id')
        measures = pilot_dict.get('measures')

        if not measures:
            return Pilot(record_key=record_id, record_id=record_id)

        name = measures.get('17')
        pilot = measures.get('16')
        description = measures.get('18')
        return Pilot(record_key=record_id, record_id=record_id, name=name, pilot=pilot, description=description)

    def get_template(self):
        return {
            "key": "{}".format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
            "pilot": "",
            "description": ""
        }

    def to_dict(self):
        result = {
            "key": "{}".format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
        }

        if self.get_pilot() is not None:
            result["pilot"] = self.get_pilot()

        if self.get_description() is not None:
            result["description"] = self.get_description()

        return result

    def get_pilot(self):
        return self.__pilot

    def get_description(self):
        return self.__description



import re
import os
import requests

from urllib.parse import urljoin
from .. import constants


class Pybrary:
    __instance = None

    @staticmethod
    def get_instance(
            username=None,
            password=None,
            superuser=False,
            base_url=constants.BASE_URL,
            base_api_url=constants.BASE_API_URL
    ):
        if Pybrary.__instance is None:
            Pybrary(username, password, superuser, base_url, base_api_url)
        return Pybrary.__instance

    def __init__(
            self,
            username=None,
            password=None,
            superuser=False,
            base_url=constants.BASE_URL,
            base_api_url=constants.BASE_API_URL
    ):
        if Pybrary.__instance is not None:
          raise Exception(
            'You are already logged in as {}, call Pybrary.get_instance().'.format(username))
        try:
            self.__base_url = base_url
            self.__base_api_url = base_api_url
            if username and password:
                self.__session = self.__login(username, password, superuser)
            else:
                self.__session = requests.Session()

            Pybrary.__instance = self
        except AttributeError as e:
            raise Exception("Error while logging {}".format(e))

    def __login(self, username, password, superuser):
        """
        Login to Databrary
        :param username: a valid user name (email)
        :param password: Databrary password
        :return: a session
        """
        session = requests.Session()
        url = urljoin(self.__base_api_url, 'user/login')
        credentials = {
            "email": username,
            "password": password,
            "superuser": superuser
        }

        response = session.post(url=url, json=credentials)
        if response.status_code == 200:
            response_json = response.json()
            if 'csverf' in response_json:
                session.headers.update({
                    "x-csverf": response_json['csverf']
                })
        else:
            raise AttributeError(
                'Login failed, please check your username and password')

        return session

    def get_request_session(self):
        """
          Return the request session
        """
        return self.__session

    def logout(self):
        """
        Disconnect from Databrary
        :return:
        """
        url = urljoin(self.__base_api_url, 'user/logout')
        response = self.__session.post(url=url)
        if response.status_code == 200:
            Pybrary.__instance = None
            __username = None
            __supersuer = None
            del self.__session
        else:
            raise AttributeError(
                'Login failed, please check your username and password')

    def get_party(self, party_id):
        """
        Get user info
        :param party_id: user id
        :return: user info in json format
        """
        url = urljoin(self.__base_api_url, 'party/' + str(party_id))
        response = self.__session.get(url=url)
        if response.status_code == 200:
            return response.json()
        else:
            raise AttributeError('Cannot fetch party data')

    def get_csv(self, volume_id, target_dir):
        """
        Download a CSV file from a Databrary volume, read access to the volume is required.
        :param volume_id: Databrary volume id
        :param target_dir: CSV file directory target
        :return: Path to the CSV file
        """

        def get_filename_from_cd(cd):
            """
            Get filename from content-disposition
            """
            if not cd:
                return None
            fname = re.findall('filename="(.+)"', cd)
            if len(fname) == 0:
                return None
            return fname[0]

        url = urljoin(self.__base_url, 'volume/' + str(volume_id) + '/csv')

        response = self.__session.get(url, allow_redirects=True)
        if response.status_code == 200:
            file_name = get_filename_from_cd(
                response.headers.get('content-disposition'))
            file_path = os.path.join(target_dir, file_name)
            open(file_path, 'wb').write(response.content)
            return file_path
        else:
            raise AttributeError(
                'Cannot download CSV file from volume %d', volume_id)

    def get_session_records(self, volume_id, session_id):
        """
        Get session records
        :param volume_id: volume id
        :param session_id: session id
        :return:
        """
        payload = {'records': 1}
        url = urljoin(self.__base_api_url, 'volume/' +
                      str(volume_id) + '/slot/' + str(session_id))

        response = self.__session.get(url=url, params=payload)
        if response.status_code == 200:
            return response.json()['records']
        else:
            raise AttributeError(
                'Cannot retrieve records list from session %d in volume %d', session_id, volume_id)

    def get_session_participants(self, volume_id, session_id):
        """
        Get session participants
        :param volume_id: Volume id
        :param session_id: Session id
        :return:
        """
        records = self.get_session_records(volume_id, session_id)
        participants_list = [record for record in records if record.get(
            "record", {}).get("category") == 1]
        return participants_list

    def get_session_assets(self, volume_id, session_id):
        """
        Get volume's asset list
        :param volume_id: Databrary volume id
        :param session_id: Databrary session id
        :return: a list of session ids in JSON format
        """
        payload = {'assets': 1}
        url = urljoin(self.__base_api_url, 'volume/' +
                      str(volume_id) + '/slot/' + str(session_id))

        response = self.__session.get(url=url, params=payload)
        if response.status_code == 200:
            return response.json()['assets']
        else:
            raise AttributeError(
                'Cannot retrieve asset list from session %d in volume %d', session_id, volume_id)

    def get_sessions(self, volume_id):
        """
        Get a list of containers(session) from a Databrary volume
        :param volume_id: Databrary volume id
        :return: a list of session ids in JSON format
        """
        payload = {'containers': 1}
        url = urljoin(self.__base_api_url, 'volume/' + str(volume_id))

        response = self.__session.get(url=url, params=payload)
        if response.status_code == 200:
            return response.json()['containers']
        else:
            raise AttributeError(
                'Cannot retrieve sessions list from volume %d', volume_id)

    def get_volume_assets(self, volume_id):
        """
        Get volume assets, the function will fetch assets found in the volume sessions
        :param volume_id: Volume id
        :return: a list of assets in json format
        """
        sessions = []
        for session in self.get_sessions(volume_id):
            session.update(
                {"assets": self.get_session_assets(volume_id, session['id'])})
            sessions.append(session)
        return sessions

    def get_volume_info(self, volume_id):
        """
        Get volume's info data
        Example:
        {
          "id": 1,
          "name": "Volume name",
          "body": "Volume description",
          "alias": "Volume alias",
          "creation": "2014-11-17T19:08:27.939187Z",
          "owners": [
            {
              "name": "Nezzar, Reda",
              "id": 4291
            },
            ...
          ],
          "permission": 5,
          "publicsharefull": null,
          "publicaccess": "none",
          "access": [
            {
              "individual": 5,
              "children": 0,
              "party": {
                "id": 4291,
                "sortname": "Nezzar",
                "prename": "Reda",
                "affiliation": "NYU",
                "email": "email@nyu.edu",
                "permission": 5
              }
            }
          ],
          "citation": null,
          "links": [],
          "funding": [
            {
              "funder": {
                "id": 1,
                "name": "Databrary"
              },
              "awards": []
            }
          ],
          "tags": [],
          "excerpts": [],
          "comments": [],
          "records": [
            {
              "id": 1,
              "category": 1,
              "measures": {
                "1": "1231",
                "4": "2014-05-01",
                "5": "Female",
                "6": "Unknown or not reported",
                "7": "Not Hispanic or Latino",
                "11": "typical",
                "12": "English"
              }
            },
            {
              "id": 2,
              "category": 2,
              "measures": {}
            },
            {
              "id": 3,
              "category": 3,
              "measures": {
                "21": "Withdrew/fussy/tired"
              }
            },
            {
              "id": 4,
              "category": 4,
              "measures": {
                "23": "Crawler"
              }
            },
            {
              "id": 5,
              "category": 5,
              "measures": {
                "26": "12 month olds"
              }
            },
            {
              "id": 7,
              "category": 7,
              "measures": {
                "33": "Lab",
                "35": "US",
                "36": "NY"
              }
            },
          ],
          "containers": [
            {
              "id": 1,
              "top": true,
              "records": [],
              "assets": []
            },
            {
              "id": 2,
              "date": "2014-11-17",
              "release": 2,
              "records": [
                {
                  "id": 1,
                  "age": 511
                },
                {
                  "id": 2
                },
                {
                  "id": 3
                },
                {
                  "id": 4
                },
                {
                  "id": 5,
                  "segment": [
                    0,
                    null
                  ]
                }
              ],
              "assets": [
                {
                  "id": 1,
                  "format": -800,
                  "duration": 1925035,
                  "segment": [
                    0,
                    1925035
                  ],
                  "name": "asset_name",
                  "permission": 5,
                  "size": 659371898
                }
              ]
            },
          ],
          "metrics": [],
          "state": {}
        }
        :param volume_id:  Volume id
        :return: volume's info in json format
        """
        payload = {
            'access': "",
            'citation': "",
            'links': "",
            'funding': "",
            'top': "",
            'tags': "",
            'excerpts': "",
            'comments': "",
            'records': "",
            'containers': "all",
            'metrics': "",
            'state': "",
        }

        url = urljoin(self.__base_api_url, 'volume/' + str(volume_id))

        response = self.__session.get(url=url, params=payload)
        if response.status_code == 200:
            return response.json()
        else:
            raise AttributeError('Cannot retrieve volume %d info', volume_id)

    def get_asset_info(self, asset_id):
        """
        Get asset Id
        :param asset_id:
        :return:
        """
        url = urljoin(self.__base_api_url, 'asset/' + str(asset_id))

        response = self.__session.get(url=url)
        if response.status_code == 200:
            return response.json()
        else:
            raise AttributeError('Cannot retrieve asset %d info', asset_id)

    def post_asset_name(self, asset_id, asset_name):
        """
        Change asset name
        :param asset_id: Asset id
        :param asset_name: New asset name
        :return:
        """
        payload = {'name': str(asset_name)}
        url = urljoin(self.__base_api_url, 'asset/' + str(asset_id))

        response = self.__session.post(url=url, json=payload)
        if response.status_code == 200:
            return response.json()
        else:
            raise AttributeError(
                'Cannot change asset name to %s', str(asset_name))

    def post_asset_permission(self, asset_id, permission):
        """
        Change asset permissions, permissions are:
            Private = 0
            Authorized = 1
            Learning = 2
            Public = 3
        :param asset_id: asset id
        :param permission: permission id
        :return:
        """
        payload = {
            'name': self.get_volume_info(asset_id)['name'],
            'classification': permission
        }
        url = urljoin(self.__base_api_url, 'asset/' + str(asset_id))
        response = self.__session.post(url=url, json=payload)
        if response.status_code == 200:
            return response.json()
        else:
            raise AttributeError(
                'Cannot change asset permission to %s', str(permission))



from enum import Enum


class Gender(Enum):
    FEMALE = 'Female'
    MALE = 'Male'

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        if not cls.has_value(value):
            return None

        return cls._value2member_map_[value]



from enum import Enum


class Ethnicity(Enum):
    NOT_HISPANIC_OR_LATINO = 'Not Hispanic or Latino'
    HISPANIC_OR_LATINO = 'Hispanic or Latino'

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        if not cls.has_value(value):
            return None

        return cls._value2member_map_[value]



from enum import Enum


class Reason(Enum):
    DID_NOT_MEET_INCLUSION_CRITERIA = 'Did not meet inclusion criteria'
    PROCEDURAL_EXPERIMENTER_ERROR = 'Procedural/experimenter error'
    WITHDREW_FUSSY_TIRED = 'Withdrew/fussy/tired'
    OUTLIER = 'Outlier'

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        if not cls.has_value(value):
            return None

        return cls._value2member_map_[value]



import warnings

from .. import constants

from .recordFactory import RecordFactory
from .record import Record
from .participant import Participant
from .asset import Asset
from .types.release import Release


class Container:
    def __init__(
            self,
            container_id,
            container_key,
            name=None,
            date='',
            top=False,
            release=Release.PRIVATE,
            assets=None,
            records=None,
            volume=None
    ):
        """
        :param container_id:
        :param container_key: 
        :param name: 
        :param date: 
        :param top: 
        :param release:
        :param assets: A hashmap of asset objects <Asset ID, Asset Object>
        :param records: A hashmap of record object<Record ID, Record Object>
        """
        if container_id is None:
            raise Exception('A valid id is required for containers')

        self.__id = container_id
        self.__key = container_key
        self.__top = top
        self.__name = name
        self.__release = release
        self.__date = date if len(date) > 0 else None
        self.__volume = volume
        self.__init_assets(assets)
        self.__init_records(records)

    def __init_records(self, records):
        self.__records = records if records is not None else {}
        for record in self.__records.values():
            record.set_container(self)

    def __init_assets(self, assets):
        self.__assets = assets if assets is not None else {}
        for asset in self.__assets.values():
            asset.set_container(self)

    def set_volume(self, volume):
        self.__volume = volume

    def get_volume(self):
        return self.__volume

    def get_key(self) -> int:
        return self.__key

    def get_id(self) -> int:
        return self.__id

    def get_assets(self) -> {int, Asset}:
        return self.__assets

    def set_assets(self, assets: {int, Asset}):
        """
        Set a hashmap of assets <Asset id, Asset Obj> to this container
        :param assets:
        :return:
        """
        self.__init_assets(assets)

    def get_records(self) -> {int, Record}:
        return self.__records

    def set_records(self, records: {int, Record}):
        self.__init_records(records)

    def add_asset(self, asset: Asset):
        """
        Add an Asset Object to the asset hashmap
        :param asset:
        :return:
        """
        asset_id = asset.get_id()
        if self.asset_id_exists(asset_id):
            raise Exception('Asset id {} already exists in container {}'.format(asset.get_id(), self.__id))

        self.__assets[asset_id] = asset
        asset.set_container(self)

    def add_record(self, record: Record):
        """
        Add a Record object to the records hashmap
        :param record:
        :return:
        """
        record_id = record.get_id()
        if self.record_id_exists(record_id):
            raise Exception('Record id {} already exists in container {}'
                            .format(record.get_id(), self.__id))

        if isinstance(record, Participant):
            if self.participant_id_exists(record.get_participant_id()):
                raise Exception('Participant id {} already exists in container {}'
                                .format(record.get_participant_id(), self.__id))

        self.__records[record_id] = record
        record.set_container(self)

    def record_id_exists(self, record_id: int):
        if record_id in self.__records:
            return True

        return False

    def participant_id_exists(self, participant_id: int):
        if self.get_participant(participant_id) is not None:
            return True

        return False

    def asset_id_exists(self, asset_id: int):
        if asset_id in self.__assets:
            return True

        return False

    def get_name(self) -> str:
        return self.__name

    def set_name(self, value: str):
        if value is None:
            raise Exception('You must provide a valid name for container {}'.format(self.__id))

        self.__name = value

    def get_asset(self, asset_id: int) -> Asset:
        return self.__assets.get(asset_id)

    def get_record(self, record_id: int) -> Record:
        return self.__records.get(record_id)

    def get_top(self) -> bool:
        return self.__top

    def set_release(self, release: Release):
        self.__release = release

    def get_release(self) -> Release:
        return self.__release

    def get_date(self) -> str:
        return self.__date

    def get_participants(self) -> {int, Participant}:
        participant_records = dict(
            filter(lambda elem: isinstance(elem[1], Participant),
                   self.__records.items()))
        participant_records = {participant.get_participant_id(): participant
                               for record_id, participant in participant_records.items()}
        return participant_records

    def get_participant(self, participant_id: str) -> Participant:
        return self.get_participants().get(participant_id)

    def remove_participant(self, participant_id: str):
        participant = self.get_participant(participant_id)
        if participant is not None:
            self.__records.pop(participant.get_id(), None)

    @staticmethod
    def from_dict(container_dict):
        container_id = container_dict.get('key')
        name = container_dict.get('name')
        assets = container_dict.get('assets')
        assets = {asset_dict.get('id'): Asset.from_dict(asset_dict) for asset_dict in assets}
        records = container_dict.get('records')
        records = {record_dict.get('key'): RecordFactory.from_dict(record_dict) for record_dict in records}
        release = Release[container_dict.get('release')]
        date = container_dict.get('date')
        top = container_dict.get('top')

        return Container(
            container_key=container_id,
            container_id=container_id,
            name=name,
            release=release,
            top=top,
            date=date,
            assets=assets,
            records=records
        )

    @staticmethod
    def from_databrary(container_dict):
        container_id = container_dict.get('id')
        date = container_dict.get('date')
        top = container_dict.get('top') is not None
        name = container_dict.get('name')

        records = dict()
        for record_dict in container_dict.get('records'):
            record_id = record_dict.get('id')
            try:
                records[record_id] = RecordFactory.from_databrary(record_dict)
            except Exception as e:
                warnings.warn('Container {} Error: {}'.format(container_id, e))

        assets = dict()
        for asset_dict in container_dict.get('assets'):
            asset_id = asset_dict.get('id')
            try:
                path_prefix = "{}/{}".format(constants.DEFAULT_SERVER_PREFIX, container_id)
                assets[asset_id] = Asset.from_databrary(asset_dict, path_prefix)
            except Exception as e:
                warnings.warn('Container {} Error: {}'.format(container_id, e))

        return Container(
            container_key=container_id,
            container_id=container_id,
            name=name,
            top=top,
            date=date,
            assets=assets,
            records=records
        )

    def get_template(self):
        return {
            "key": "{}".format(self.get_key()),
            "top": self.get_top(),
            "release": self.get_release().value,
            "name": "",
            "date": "",
            "assets": [],
            "records": []
        }

    def to_dict(self):
        result = {
            "key": "{}".format(self.get_key()),
            "top": self.get_top(),
            "release": self.get_release().value,
        }

        if self.get_name() is not None:
            result['name'] = self.get_name()

        if self.get_date() is not None:
            result['date'] = self.get_date()

        result['assets'] = list(map(lambda asset: asset.to_dict(), list(self.get_assets().values())))
        result['records'] = list(map(lambda record: record.to_dict(), list(self.get_records().values())))

        return result

    def __eq__(self, other):
        return self.__id == other.get_id()



import os

BASE_API_URL = 'https://nyu.databrary.org/api/'
BASE_URL = 'https://nyu.databrary.org/'
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
VOLUME_SCHEMA_FILE = os.path.join(ROOT_DIR, 'volume.json')
DEFAULT_SERVER_PREFIX = '/nyu/stage/reda'

VIDEO_EXTENSIONS = ["webm", "mpg",
                    "mp4", "mov",
                    "mts", "avi",
                    "wmv", "dv"]

AUDIO_EXTENSIONS = ["wav", "aac",
                    "wma", "mp3"]

OPF_EXTENSION = ["opf"]

DATABRARY_VOLUME_CATEGORIES = {
    1: "participant",
    2: "pilot",
    3: "exclusion",
    4: "condition",
    5: "group",
    6: "task",
    7: "context"
}

DATABRARY_CLASSIFICATIONS = {
    0: 'private',
    1: 'shared',
    2: 'excerpt',
    3: 'public'
}

SUPPORTED_FORMATS = {
    2: "csv",
    4: "rtf",
    5: "png",
    6: "pdf",
    7: "doc",
    8: "odf",
    9: "docx",
    10: "xls",
    11: "ods",
    12: "xlsx",
    13: "ppt",
    14: "odp",
    15: "pptx",
    16: "opf",
    18: "webm",
    20: "mov",
    -800: "mp4",
    22: "avi",
    23: "sav",
    24: "wav",
    19: "mpeg",
    26: "chat",
    -700: "jpeg",
    21: "mts",
    -600: "mp3",
    27: "aac",
    28: "wma",
    25: "wmv",
    29: "its",
    30: "dv",
    1: "txt",
    31: "etf"
}



from .volume import Volume
from .asset import Asset
from .recordFactory import RecordFactory
from .condition import Condition
from .context import Context
from .exclusion import Exclusion
from .group import Group
from .pilot import Pilot
from .task import Task
from .participant import Participant
from .container import Container

__all__ = [
    Volume,
    Container,
    Asset,
    Participant,
    RecordFactory,
    Condition,
    Context,
    Group,
    Pilot,
    Task,
    Exclusion
]



import os
import csv
import json
import pandas as pd
from .. import constants
# TODO: Finish unit tests


def replace_file_extension(file_path, file_format='json'):
    root = get_file_root(file_path)
    file_name = get_file_name_no_extension(file_path)
    return os.path.join(root, "{}{}{}".format(file_name, '.', file_format))


# TODO: Check unit test and fix edge cases
def get_file_extension(file_path):
    if file_path is None:
        return None

    file_name = get_file_name(file_path)

    file_name, file_extension = os.path.splitext(file_name)
    file_ext = file_extension[1:] if len(file_extension[1:]) > 0 else None
    return file_ext


# TODO: Check unit test and fix edge cases
def get_file_name_no_extension(file_path):
    file_name, file_extension = os.path.splitext(get_file_name(file_path))

    if len(file_name) == 0:
        return None

    return file_name


def get_file_root(file_path):
    if file_path is None:
        return None

    return os.path.dirname(os.path.abspath(file_path))


# TODO: Check unit test and fix edge cases
def get_file_name(file_path):
    if file_path is None:
        return None

    return os.path.basename(file_path)


def get_supported_formats():
    return constants.SUPPORTED_FORMATS.values()


def dump_into_csv(data_list, output, index=False):
    df = pd.read_json(json.dumps(data_list))
    df.to_csv(output, index=index, index_label='id', quoting=csv.QUOTE_ALL)


def dump_into_json(data_list, output, indent=4):
    with open(output, 'w') as outFile:
        json.dump(data_list, outFile, indent=indent)


def read_csv(file_path: str):
    df = pd.read_csv(file_path, keep_default_na=False)
    return df.to_dict('records')


def get_category_from_category_id(category_id: int) -> str:
    return constants.DATABRARY_VOLUME_CATEGORIES[category_id]


def get_databrary_ext_from_format(format_id: int) -> str:
    return constants.SUPPORTED_FORMATS[format_id]



from .record import Record
from .types.category import Category
from .types.gender import Gender
from .types.ethnicity import Ethnicity
from .types.race import Race


class Participant(Record):
    PARTICIPANT_METRICS = {
        "1": "ID",
        "2": "info",
        "3": "description",
        "4": "birthdate",
        "5": "gender",
        "6": "race",
        "7": "ethnicity",
        "8": "gestational age",
        "9": "pregnancy term",
        "10": "birth weight",
        "11": "disability",
        "12": "language",
        "13": "country",
        "14": "state",
        "15": "setting"
    }

    def __init__(
            self,
            record_key,
            record_id,
            participant_id=None,
            race=Race.UNKNOWN_OR_NOT_REPORTED,
            ethnicity=None,
            gender=None,
            birthdate=None,
            disability=None,
            language=None,
            gestational_age=None,
            birth_weight=None,
            container=None,
    ):
        super().__init__(record_key, record_id, Category.PARTICIPANT, container)

        if ethnicity and not isinstance(ethnicity, Ethnicity):
            raise Exception('ethnicity arg for record {} must be an instance of Ethnicity enum'
                            .format(record_key))

        if gender and not isinstance(gender, Gender):
            raise Exception('gender arg for record {} must be an instance of Gender enum'
                            .format(record_key))

        self.__participant_id = participant_id
        self.__race = race
        self.__ethnicity = ethnicity
        self.__gender = gender
        self.__birthdate = birthdate
        self.__disability = disability
        self.__language = language
        self.__gestational_age = gestational_age
        self.__birth_weight = birth_weight

    @staticmethod
    def from_dict(participant_dict):
        record_id = participant_dict.get('key')
        participant_id = participant_dict.get('ID')
        race = Race.get_name(participant_dict.get('race'))
        ethnicity = Ethnicity.get_name(participant_dict.get('ethnicity'))
        gender = Gender.get_name(participant_dict.get('gender'))
        birthdate = participant_dict.get('birthdate')
        disability = participant_dict.get('disability')
        language = participant_dict.get('language')
        gestational_age = participant_dict.get('gestational_age')
        birth_weight = participant_dict.get('birth_weight')

        return Participant(
            record_key=record_id,
            record_id=record_id,
            participant_id=participant_id,
            race=race,
            ethnicity=ethnicity,
            gender=gender,
            birthdate=birthdate,
            disability=disability,
            language=language,
            gestational_age=gestational_age,
            birth_weight=birth_weight
        )

    @staticmethod
    def from_databrary(participant_dict):
        # TODO: Get measures from the hash map
        record_id = participant_dict.get('id')
        measures = participant_dict.get('measures')

        if not measures:
            return Participant(record_key=record_id, record_id=record_id)

        race = Race.get_name(measures.get('6'))
        participant_id = int(measures.get('1'))
        ethnicity = Ethnicity.get_name(measures.get('7'))
        gender = Gender.get_name(measures.get('5'))
        birthdate = measures.get('4')
        disability = measures.get('11')
        language = measures.get('12')
        gestational_age = measures.get('8')
        birth_weight = measures.get('10')

        return Participant(
            record_key=record_id,
            record_id=record_id,
            participant_id=participant_id,
            race=race,
            ethnicity=ethnicity,
            gender=gender,
            birthdate=birthdate,
            disability=disability,
            language=language,
            gestational_age=gestational_age,
            birth_weight=birth_weight
        )

    def get_template(self):
        return {
            "key": "{}".format(self.get_key()),
            "ID":  "{}".format(self.get_participant_id() if self.get_participant_id() is not None else self.get_key()),
            "category": self.get_category().value,
            "birthdate": "YYYY-MM-DD",
            "language": "",
            "disability": "",
            "gender": "",
            "race": "",
            "ethnicity": ""
        }

    def to_dict(self):
        result = {
            "key": "{}".format(self.get_key()),
            "ID": "{}".format(self.get_participant_id() if self.get_participant_id() is not None else self.get_id()),
            "category": self.get_category().value,
        }

        if self.get_birthdate() is not None:
            result['birthdate'] = self.get_birthdate()
        if self.get_language() is not None:
            result['language'] = self.get_language()
        if self.get_disability() is not None:
            result['disability'] = self.get_disability()
        if self.get_gender() is not None:
            result['gender'] = self.get_gender().value
        if self.get_race() is not None:
            result['race'] = self.get_race().value
        if self.get_ethnicity() is not None:
            result['ethnicity'] = self.get_ethnicity().value

        return result

    def get_race(self):
        return self.__race

    def get_ethnicity(self):
        return self.__ethnicity

    def get_gender(self):
        return self.__gender

    def get_birthdate(self):
        return self.__birthdate

    def get_disability(self):
        return self.__disability

    def get_language(self):
        return self.__language

    def get_participant_id(self):
        return self.__participant_id

    def get_gestational_age(self):
        return self.__gestational_age

    def get_birth_weight(self):
        return self.__birth_weight

    def __eq__(self, other):
        return super().__eq__(other) or self.__participant_id == other.get_participant_id()



from .record import Record
from .types.category import Category


class Task(Record):
    TASK_METRICS = {
        "29": "name",
        "30": "description",
        "31": "info"
    }

    def __init__(self, record_key, record_id, name=None, info=None, description=None, container=None):
        super().__init__(record_key, record_id, Category.TASK, name=name, container=container)
        self.__info = info
        self.__description = description

    @staticmethod
    def from_dict(task_dict):
        record_id = task_dict.get('key')
        name = task_dict.get('name')
        info = task_dict.get('info')
        description = task_dict.get('description')

        return Task(record_key=record_id, record_id=record_id, name=name, info=info, description=description)

    @staticmethod
    def from_databrary(task_dict):
        record_id = task_dict.get('id')
        measures = task_dict.get('measures')

        if not measures:
            return Task(record_key=record_id, record_id=record_id)

        name = measures.get('29')
        info = measures.get('31')
        description = measures.get('30')

        return Task(
            record_key=record_id,
            record_id=record_id,
            name=name,
            info=info,
            description=description
        )

    def get_template(self):
        return {
            "key": '{}'.format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
            "info": "",
            "description": ""
        }

    def to_dict(self):
        result = {
            "key": '{}'.format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
        }

        if self.get_info() is not None:
            result["info"] = self.get_info()

        if self.get_description() is not None:
            result["description"] = self.get_description()

        return result

    def get_info(self):
        return self.__info

    def get_description(self):
        return self.__description



from .types.category import Category
from ..utils import utils

from .participant import Participant
from .pilot import Pilot
from .group import Group
from .exclusion import Exclusion
from .context import Context
from .condition import Condition
from .task import Task


class RecordFactory:
    @staticmethod
    def from_dict(record_dict):
        category = Category.get_name(record_dict.get('category'))
        if category is None:
            raise Exception('A category key is required in the record dictionary')

        record = RecordFactory.get_record_class(category.value)
        return record.from_dict(record_dict)

    @staticmethod
    def from_databrary(record_dict):
        category = record_dict.get('category')
        if not isinstance(category, int):
            raise Exception("Databrary's record categories must be numeric")

        category = utils.get_category_from_category_id(category)

        if not Category.has_value(category):
            raise Exception('Cannot find category {}'.format(category))

        record = RecordFactory.get_record_class(category)
        return record.from_databrary(record_dict)

    @staticmethod
    def get_record_class(category: str):
        if category == "participant":
            return Participant
        elif category == "group":
            return Group
        elif category == "pilot":
            return Pilot
        elif category == "exclusion":
            return Exclusion
        elif category == "context":
            return Context
        elif category == "task":
            return Task
        elif category == "condition":
            return Condition



from enum import Enum


class Setting(Enum):
    LAB = 'Lab'
    HOME = 'Home' 
    CLASSROOM = 'Classroom' 
    OUTDOOR = 'Outdoor' 
    CLINIC = 'Clinic'

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        if not cls.has_value(value):
            return None

        return cls._value2member_map_[value]



import warnings
import json

from jsonschema import validate, ValidationError

from .container import Container
from .. import constants


class Volume:
    """
    A volume is a set is a set of containers(aka Session), assets and record, A container might point to
    assets and records attached to it.
    The volume class is in charge to fetch the correct asset and record from assets and records list respectively
    and generate an ingest dictionary to be processed by the Curation class
    """
    def __init__(self, volume_name: str, containers=None):
        """
        Create and instance of a Databrary volume
        :param volume_name: Volume's name | Required
        :param containers: A hashmap of container <Container Id, container obj>
        """
        if volume_name is None or len(volume_name) < 1:
            raise AttributeError('You must provide a valid volume name.')

        self.__name = volume_name
        self.__containers = containers

    @staticmethod
    def from_dict(volume_name, containers_map):
        containers = dict()
        for key, container_dict in containers_map.items():
            containers[key] = Container.from_dict(container_dict)

        return Volume(volume_name, containers)

    @staticmethod
    def from_databrary(volume_dict):
        """
        Parse Volume dictionary and populate containers, assets and records (if Any)
        The method will link assets and records to the appropriate container as well
        :param volume_dict: Volume's JSON file
        :return:
        """
        containers = dict()

        # Databrary volume have records in a separate list
        # the record id is referenced in the container records list
        # We need to fetch records before creating our list of containers
        records = dict()
        for record_dict in volume_dict.get('records'):
            record_id = record_dict.get('id')

            if record_id in records:
                raise Exception('Duplicate Record id {}'.format(record_id))

            if record_id is not None:
                records[record_id] = record_dict

        for container_dict in volume_dict.get('containers'):
            container_records = []

            for container_record in container_dict.get('records'):
                record_id = container_record.get('id')
                record_dict = records.get(record_id)
                if record_dict is None:
                    warnings.warn("Cannot find record {} id in the volume's records list".format(record_id))
                else:
                    container_records.append(record_dict)

            container_dict['records'] = container_records
            try:
                container = Container.from_databrary(container_dict)
                containers[container.get_id()] = container
            except Exception as e:
                warnings.warn("Volume {} Error: {}".format(volume_dict.get('name'), e))

        return Volume(
            volume_name=volume_dict.get('name'),
            containers=containers
        )

    def get_containers(self) -> {int, Container}:
        return self.__containers

    def get_container(self, container_id: int) -> Container:
        return self.__containers.get(container_id)

    def remove_participant(self, participant_id: str):
        container = self.get_container_of_participant(participant_id)
        container.remove_participant(participant_id)

    def remove_container(self, container_id: int):
        self.__containers.pop(container_id, None)

    def to_dict(self):
        result = {
            "name": self.get_name(),
        }

        container_list = []
        for container in self.__containers.values():
            container_dict = container.to_dict()
            container_list.append(container_dict)

        result['containers'] = container_list
        # Volume.validate(result)

        return result

    def to_json(self, indent=4):
        return json.dumps(self.to_dict(), indent=indent)

    def get_container_of_record(self, record_id: int):
        for container in self.__containers.values():
            if record_id in container.get_records():
                return container

        return None

    def get_container_of_participant(self, participant_id: str):
        for container in self.__containers.values():
            if container.get_participant(participant_id) is not None:
                return container

        return None

    def get_container_of_asset(self, asset_id: int):
        for container in self.__containers.values():
            if asset_id in container.get_assets():
                return container

        return None

    def get_asset(self, asset_id: int):
        for container in self.__containers.values():
            asset = container.get_assets().get(asset_id)
            if asset is not None:
                return asset

        return None

    def get_record(self, record_id: int):
        for container in self.__containers.values():
            record = container.get_records().get(record_id)
            if record is not None:
                return record

        return None

    def get_name(self) -> str:
        return self.__name

    def get_assets(self):
        assets = dict()
        for container in self.__containers.values():
            for asset in container.get_assets().values():
                if asset.get_id() in assets:
                    raise Exception("Found duplicate asset id {} in volume".format(asset.get_id()))

                assets[asset.get_id()] = asset

        return assets

    def get_records(self):
        records = dict()
        for container in self.__containers.values():
            for record in container.get_records().values():
                if record.get_id() in records:
                    raise Exception("Found duplicate record id {} in volume".format(record.get_id()))

                records[record.get_id()] = record

        return records

    @staticmethod
    def validate(volume_data, schema_file=constants.VOLUME_SCHEMA_FILE):
        with open(schema_file) as f:
            schema = json.loads(f.read())
        try:
            validate(volume_data, schema)
        except ValidationError as e:
            raise Exception('Did not pass validation against volume.json schema - Errors: {}'.format(e))



from .types.release import Release
from ..utils import utils
from .. import constants


class Asset:
    def __init__(
            self,
            file_path,
            asset_id,
            container=None,
            name=None,
            position='auto',
            release=Release.PRIVATE,
            clips=None
    ):
        if asset_id is None:
            raise Exception('Asset id is required')

        if file_path is None:
            raise Exception('You must provide a file path to the asset {}.'.format(asset_id))

        self._file_ext = utils.get_file_extension(file_path)
        if not Asset.is_ext_supported(self._file_ext):
            raise Exception('Asset format {} not supported'.format(self._file_ext))

        if not Asset.is_media(self._file_ext) and clips is not None:
            raise Exception('Cannot add clips for {}. Clips are only supported for media types'.format(self._file_ext))

        self.__clips = clips
        self.__id = asset_id
        self.__file_path = file_path
        self.__name = utils.get_file_name(file_path) if name is None else name
        self.__position = position
        self.__release = release
        self.__container = container

    def get_container(self):
        return self.__container

    def set_container(self, container):
        self.__container = container

    def get_id(self) -> int:
        return self.__id

    def get_name(self) -> str:
        return self.__name

    def set_name(self, value: str):
        self.__name = value

    def get_release(self) -> Release:
        return self.__release

    def set_release(self, release: Release):
        self.__release = release

    @staticmethod
    def from_dict(asset_dict):
        file = asset_dict.get('file')
        asset_id = asset_dict.get('id')
        name = asset_dict.get('name')
        return Asset(asset_id=asset_id, file_path=file, name=name)

    @staticmethod
    def from_databrary(asset_dict, path_prefix=constants.DEFAULT_SERVER_PREFIX):
        """
        Building an Asset Object from Databrary Asset Dict fetched from the API.
        Important: Pybrary cannot know the file name, so it will build the file path
        from the Asset's name, Make sure that your file names and asset
        names are the same
        if not, please contact you Databrary Administrator
        Example: {
          "id": 12094,
          "format": -800,
          "duration": 1925035,
          "segment": [
            0,
            1925035
          ],
          "name": "InfantOHApS#74",
          "permission": 5,
          "size": 659371898
        }
        :param asset_dict:
        :param path_prefix:
        :return:
        """
        asset_id = asset_dict.get('id')

        file_name = utils.get_file_name_no_extension(asset_dict.get('name'))
        file_ext = utils.get_databrary_ext_from_format(asset_dict.get('format'))

        if not file_name:
            raise Exception("Asset {} does not contain a file name".format(asset_id))

        file_name = file_name

        if file_ext is None:
            raise Exception('Format {} of Asset id {} is not supported by Databrary'
                            .format(asset_dict.get('format'), asset_id))

        file_name_ext = "{}.{}".format(file_name, file_ext)

        file_path = "{}/{}".format(path_prefix, file_name_ext)
        return Asset(file_path=file_path, asset_id=asset_id, name=file_name)

    def get_template(self):
        return {
            "id": self.get_id(),
            "release": self.get_release().value,
            "position": self.__position,
            "name": self.get_name(),
            "file": self.__file_path,
            "clips": []
        }

    def to_dict(self):
        result = {
            "id": self.get_id(),
            "release": self.get_release().value,
            "position": self.__position,
            "name": self.get_name(),
            "file": self.__file_path
        }

        if self.__clips is not None:
            result['clip'] = self.__clips

        return result

    @staticmethod
    def is_media(file_ext: str) -> bool:
        if file_ext in constants.VIDEO_EXTENSIONS \
                or file_ext in constants.AUDIO_EXTENSIONS:
            return True

        return False

    @staticmethod
    def is_ext_supported(file_ext: str) -> bool:
        if file_ext in constants.SUPPORTED_FORMATS.values():
            return True

        return False

    def __eq__(self, other):
        return self.__id == other.get_id()



from .record import Record

from .types.category import Category
from .types.setting import Setting


class Context(Record):
    CONTEXT_METRICS = {
        "32": "name",
        "33": "setting",
        "34": "language",
        "35": "country",
        "36": "state"
    }

    # TODO: Add State and country types
    def __init__(
            self,
            record_key,
            record_id,
            name=None,
            language=None,
            setting=None,
            country=None,
            state=None,
            container=None
    ):
        super().__init__(record_key, record_id, Category.CONTEXT, name=name, container=container)

        if setting and not isinstance(setting, Setting):
            raise Exception('setting arg for record {} must be an instance of Setting enum'
                            .format(record_key))

        self._setting = setting
        self._country = country
        self._state = state
        self._language = language

    @staticmethod
    def from_dict(context_dict):
        record_id = context_dict.get('key')
        setting = Setting.get_name(context_dict.get('setting'))
        country = context_dict.get('country')
        state = context_dict.get('state')
        name = context_dict.get('name')
        language = context_dict.get('language')

        return Context(
            record_key=record_id,
            record_id=record_id,
            name=name,
            language=language,
            setting=setting,
            country=country,
            state=state
        )

    @staticmethod
    def from_databrary(context_dict):
        rexord_id = context_dict.get('id')
        measures = context_dict.get('measures')

        if not measures:
            return Context(record_key=rexord_id, record_id=rexord_id)

        setting = Setting.get_name(measures.get('33'))
        country = measures.get('35')
        state = measures.get('36')
        name = measures.get('32')
        language = measures.get('34')

        return Context(
            record_key=rexord_id,
            record_id=rexord_id,
            name=name,
            language=language,
            setting=setting,
            country=country,
            state=state
        )

    def get_template(self):
        return {
            "key": "{}".format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
            "language": None,
            "setting": None,
            "state": None,
            "country": None
        }

    def to_dict(self):
        result = {
            "key": "{}".format(self.get_key()),
            "name": self.get_name(),
            "category": self.get_category().value,
        }

        if self.get_language() is not None:
            result["language"] = self.get_language()

        if self.get_setting() is not None:
            result["setting"] = self.get_setting().value

        if self.get_state() is not None:
            result["state"] = self.get_state()

        if self.get_country() is not None:
            result["country"] = self.get_country()

        return result

    def get_language(self):
        return self._language

    def get_setting(self):
        return self._setting

    def get_state(self):
        return self._state

    def get_country(self):
        return self._country



from .types.category import Category

class Record:
    def __init__(
            self,
            record_key,
            record_id,
            category,
            name=None,
            container=None
    ):
        if record_key is None or record_id is None:
            raise Exception('A valid id and key are required for a Record')

        if category is None:
            raise Exception('A category is required for a Record')

        if not isinstance(category, Category):
            raise Exception('category arg for record {} must be an instance of Category enum'
                            .format(record_key))

        self.__key = int(record_key)
        self.__id = int(record_id)
        self.__category = category
        self.__name = "{} {}".format(category.value, record_key) if name is None else name
        self.__container = container

    def get_container(self):
        return self.__container

    def set_container(self, container):
        self.__container = container

    def get_id(self):
        return self.__id

    def get_key(self):
        return self.__key

    def get_name(self):
        return self.__name

    def get_category(self):
        return self.__category

    def __eq__(self, other):
        return self.get_id() == other.get_id()



from enum import Enum


class Race(Enum):
    AMERICAN_INDIAN_OR_ALASKA_NATIVE = 'American Indian or Alaska Native'
    ASIAN = 'Asian'
    NATIVE_HAWAIIAN_OR_OTHER_PACIFIC_ISLANDER = 'Native Hawaiian or Other Pacific Islander'
    BLACK_OR_AFRICAN_AMERICAN = 'Black or African American'
    WHITE = 'White'
    MORE_THAN_ONE = 'More than one'
    UNKNOWN_OR_NOT_REPORTED = 'Unknown or not reported'

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        if value is None:
            value = 'Unknown or not reported'
        return cls._value2member_map_[value]



from enum import Enum


class Release(Enum):
    PRIVATE = "PRIVATE"
    SHARED = "SHARED"
    EXCERPTS = "EXCERPTS"
    PUBLIC = "PUBLIC"

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        return cls._value2member_map_[value]



from enum import Enum


class Category(Enum):
    PARTICIPANT = 'participant'
    PILOT = 'pilot'
    EXCLUSION = 'exclusion'
    CONDITION = 'condition'
    GROUP = 'group'
    TASK = 'task'
    CONTEXT = 'context'

    @classmethod
    def has_value(cls, value):
        return value in cls._value2member_map_

    @classmethod
    def get_name(cls, value):
        return cls._value2member_map_[value]
